"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var shared_1 = require("./shared");
var isPlainObject = function (value) {
    if (typeof value !== "object") {
        return false;
    }
    return Object.prototype.toString.call(value) === "[object Object]";
};
exports.str = function (value) {
    if (typeof value !== "string") {
        throw new shared_1.DecodeError("string", shared_1.getAccurateTypeOf(value));
    }
    return value;
};
exports.num = function (value) {
    if (typeof value !== "number") {
        throw new shared_1.DecodeError("number", shared_1.getAccurateTypeOf(value));
    }
    return value;
};
exports.bool = function (value) {
    if (typeof value !== "boolean") {
        throw new shared_1.DecodeError("boolean", shared_1.getAccurateTypeOf(value));
    }
    return value;
};
exports.nil = function (value) {
    if (value !== null) {
        throw new shared_1.DecodeError("null", shared_1.getAccurateTypeOf(value));
    }
    return value;
};
exports.array = function (decoder) { return function (values) {
    if (!Array.isArray(values)) {
        throw new shared_1.DecodeError("array", shared_1.getAccurateTypeOf(values));
    }
    values.forEach(decoder);
    return values;
}; };
exports.oneOf = function () {
    var decoders = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        decoders[_i] = arguments[_i];
    }
    return function (value) {
        var expectations = [];
        for (var _i = 0, decoders_1 = decoders; _i < decoders_1.length; _i++) {
            var decoder = decoders_1[_i];
            try {
                decoder(value);
                return value;
            }
            catch (error) {
                if (error instanceof shared_1.DecodeError) {
                    expectations = expectations.concat([error.expected]);
                }
            }
        }
        throw new shared_1.DecodeError("one of \"" + expectations.join('" or "') + "\"", shared_1.getAccurateTypeOf(value));
    };
};
exports.nullable = function (decoder) {
    return exports.oneOf(exports.nil, decoder);
};
exports.maybe = function (decoder) {
    return exports.oneOf(function (value) {
        if (value !== undefined) {
            throw new shared_1.DecodeError("undefined", shared_1.getAccurateTypeOf(value));
        }
        return value;
    }, decoder);
};
exports.union = function (decoder) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    return exports.compose(decoder, function (value) {
        if (!values.includes(value)) {
            throw new shared_1.DecodeError("one of \"" + values.join('" or "') + "\"", value);
        }
        return value;
    });
};
exports.object = function (decoders) { return function (value) {
    if (!isPlainObject(value)) {
        throw new shared_1.DecodeError("object", shared_1.getAccurateTypeOf(value));
    }
    for (var key in decoders) {
        try {
            decoders[key](value[key]);
        }
        catch (error) {
            if (error instanceof shared_1.DecodeError) {
                throw new shared_1.DecodeError(error.expected + " at field \"" + key + "\"", error.received);
            }
            throw error;
        }
    }
    return value;
}; };
exports.field = function (name, decoder) { return function (value) { return decoder(value[name]); }; };
exports.compose = function () {
    var decoders = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        decoders[_i] = arguments[_i];
    }
    return function (value) { return decoders.reduce(function (acc, reducer) { return reducer(acc); }, value); };
};
exports.map = function (f) {
    var decoders = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        decoders[_i - 1] = arguments[_i];
    }
    return function (value) { return f.apply(void 0, decoders.map(function (decoder) { return decoder(value); })); };
};
