"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var shared_1 = require("./shared");
exports.minLength = function (length) { return function (value) {
    if (typeof value !== "string") {
        throw new shared_1.DecodeError("string", shared_1.getAccurateTypeOf(value));
    }
    if (value.length <= length) {
        throw new shared_1.DecodeError("string with a length greater than " + length, value);
    }
    return value;
}; };
exports.maxLength = function (length) { return function (value) {
    if (typeof value !== "string") {
        throw new shared_1.DecodeError("string", shared_1.getAccurateTypeOf(value));
    }
    if (value.length >= length) {
        throw new shared_1.DecodeError("string with a length lesser than " + length, value);
    }
    return value;
}; };
exports.lengthWithin = function (minLength, maxLength) { return function (value) {
    if (typeof value !== "string") {
        throw new shared_1.DecodeError("string", shared_1.getAccurateTypeOf(value));
    }
    if (value.length <= minLength || value.length >= maxLength) {
        throw new shared_1.DecodeError("string of length within " + minLength + " and " + maxLength, value);
    }
    return value;
}; };
exports.min = function (minValue) { return function (value) {
    if (typeof value !== "number") {
        throw new shared_1.DecodeError("number", shared_1.getAccurateTypeOf(value));
    }
    if (value <= minValue) {
        throw new shared_1.DecodeError("number strictly greater than " + minValue, value.toString());
    }
    return value;
}; };
exports.max = function (maxValue) { return function (value) {
    if (typeof value !== "number") {
        throw new shared_1.DecodeError("number", shared_1.getAccurateTypeOf(value));
    }
    if (value >= maxValue) {
        throw new shared_1.DecodeError("number strictly lesser than " + maxValue, value.toString());
    }
    return value;
}; };
exports.within = function (minValue, maxValue) { return function (value) {
    if (typeof value !== "number") {
        throw new shared_1.DecodeError("number", shared_1.getAccurateTypeOf(value));
    }
    if (value <= minValue || value >= maxValue) {
        throw new shared_1.DecodeError("number strictly greater than " + minValue + " and strictly lesser than " + maxValue, value.toString());
    }
    return value;
}; };
exports.format = function (pattern) { return function (value) {
    if (typeof value !== "string" || !pattern.test(value)) {
        throw new shared_1.DecodeError("string matching " + pattern, value);
    }
    return value;
}; };
exports.notEmpty = function (value) {
    if (typeof value !== "string") {
        throw new shared_1.DecodeError("string", shared_1.getAccurateTypeOf(value));
    }
    if (value.length === 0) {
        throw new shared_1.DecodeError("non empty string", value);
    }
    return value;
};
